//////////////////////////////
//  When using using setSegmap, the
//      order of the arguments is:
//
//  0 5555555555555 4
//  00 55555555555 44
//  00             44
//  00             44
//  00             44
//  00             44
//  0 6666666666666 4
//  1 6666666666666 3
//  11             33
//  11             33
//  11             33
//  11             33
//  11 22222222222 33   777
//  1 2222222222222 3   777
//
//   6   5   4   3   2   1   0
//  32  16   8   4   2   1   0

extern Kernel kernel;

class SevenSeg {
    private:
        int segMap[8];
        
        int on;
        
        int digits[4];
        char disp[5];
        
        int latchPin; //ST_CP  or RCLK   (12 ada) storage register clock 
        int clockPin; //SH_CP  or SRCLK  (11 ada) shift register clock
        int dataPin;  //DS     or SER    (14 ada) serial input
        
        int curDigit;
        
        void lightDigit();
        
    public:
        SevenSeg();
    
        int getmap( char digit );
        //int getmap( int digit );
        void off();
        
        void setShift( int, int, int );
        void setDigits(int a,int b,int c,int d);
        void setSegmap(int, int, int, int, int, int, int, int);
        
        void display( char chars[5] );
        void display( int number);
        
        void setup();
};









SevenSeg::SevenSeg() {
    setShift( 8, 12, 11);
    setDigits( 7,6,5,4 );
    setSegmap( 0, 1, 2, 3, 4, 5, 6, 7);
    
    disp[0] = '1';
    disp[1] = '2';
    disp[2] = '3';
    disp[3] = '4';
    
    curDigit = 0;
}

void SevenSeg::setShift( int latch, int clock, int data ) {
    //Pin connected to ST_CP of 74HC595
    latchPin = latch;
    //Pin connected to SH_CP of 74HC595
    clockPin = clock;
    ////Pin connected to DS of 74HC595
    dataPin = data;
}
void SevenSeg::setDigits(int a,int b,int c,int d) {
    digits[0] = a;
    digits[1] = b;
    digits[2] = c;
    digits[3] = d;
}
void SevenSeg::setSegmap(int a, int b, int c, int d, int e, int f, int g, int h) {
    segMap[0] = a;
    segMap[1] = b;
    segMap[2] = c;
    segMap[3] = d;
    segMap[4] = e;
    segMap[5] = f;
    segMap[6] = g;
    segMap[7] = h;
}


void SevenSeg::setup() {
    pinMode(latchPin, OUTPUT);
    pinMode(clockPin, OUTPUT);
    pinMode(dataPin, OUTPUT);
  
    pinMode(digits[0], OUTPUT);  
    pinMode(digits[1], OUTPUT);  
    pinMode(digits[2], OUTPUT);  
    pinMode(digits[3], OUTPUT); 
    
    digitalWrite(digits[0], HIGH);
    digitalWrite(digits[1], HIGH);
    digitalWrite(digits[2], HIGH);
    digitalWrite(digits[3], HIGH);
    
    FuncDelegate f_delegate;
    f_delegate = MakeDelegate(this, &SevenSeg::lightDigit);
    kernel.setInterval( 700UL, f_delegate);
}


/*
int SevenSeg::getmap( int digit ) {
    switch (digit) {
        //////////////0b6543210
        case 0: on = 0b0111111; break;
        case 1: on = 0b0011000; break;
        case 2: on = 0b1110110; break;
        case 3: on = 0b1111100; break;
        /////////////0b6543210
        case 4: on = 0b1011001; break;
        case 5: on = 0b1101101; break;
        case 6: on = 0b1101111; break;
        case 7: on = 0b0111000; break;
        /////////////0b6543210
        case 8: on = 0b1111111; break;
        case 9: on = 0b1111101; break;
    }
    return on;
}
*/
int SevenSeg::getmap( char digit ) {
    switch (digit) {
        ///////////////0b6543210
        case '0': on = 0b0111111; break;
        case '1': on = 0b0011000; break;
        case '2': on = 0b1110110; break;
        case '3': on = 0b1111100; break;
        ///////////////0b6543210
        case '4': on = 0b1011001; break;
        case '5': on = 0b1101101; break;
        case '6': on = 0b1101111; break;
        case '7': on = 0b0111000; break;
        ///////////////0b6543210
        case '8': on = 0b1111111; break;
        case '9': on = 0b1111101; break;
        
        
        ///////////////0b6543210
        case 'a': on = 0b1111011; break;
        case 'b': on = 0b1001111; break;
        case 'c': on = 0b1000110; break;
        case 'd': on = 0b1011110; break;
        ///////////////0b6543210
        case 'e': on = 0b1100111; break;
        case 'f': on = 0b1100011; break;
        case 'g': on = 0b1111101; break;
        case 'h': on = 0b1001011; break;
        ///////////////0b6543210
        case 'i': on = 0b0001000; break;
        case 'j': on = 0b0011100; break;
        case 'l': on = 0b0000011; break;
        case 'n': on = 0b1001010; break;
        ///////////////0b6543210
        case 'o': on = 0b1001110; break;
        case 'p': on = 0b1110011; break;
        case 'q': on = 0b1111001; break;
        case 'r': on = 0b1000010; break;
        ///////////////0b6543210
        case 's': on = 0b1101101; break;
        case 'u': on = 0b0001110; break;
        case 'x': on = 0b1011011; break;
        case 'y': on = 0b1011101; break;
        case 'z': on = 0b1101101; break;
        case '-': on = 0b1000000; break;
        case ' ': on = 0b0000000; break;
    }
    return on;
}




void SevenSeg::display( char chars[5] ) {
    snprintf(disp, 5, "%4s", chars);
}
void SevenSeg::display( int number ) {
    snprintf(disp, 5, "%4d", number);
}


void SevenSeg::lightDigit() {
    digitalWrite(digits[curDigit], HIGH); // sets the digit off
    
    curDigit ++;
    if ( curDigit > 3 ) {
        curDigit = 0;
    }
    
    digitalWrite(latchPin, LOW);
    //turn on the segments
    shiftOut(dataPin, clockPin, MSBFIRST, getmap( disp[curDigit]) );
    digitalWrite(latchPin, HIGH);
    
    digitalWrite(digits[curDigit], LOW);   // sets the digit on
    
    
    
}

